/**
 * MainController
 *
 * @module		:: Controller
 * @description	:: Contains logic for handling requests.
 */
var fs = require('fs');
var converter = require('../../lib/database/converter');
var staticModels;

module.exports = {
  /**
   * Index page route /
   */
  index: function(req, res) {
    var we = req.getWe();

    res.locals.template = 'home/index';
    res.view({ title: we.config.appName });
  },

  /**
   * Client side configs
   * @param  {object} req
   * @param  {object} res
   */
  getConfigsJS: function(req, res) {
    var we = req.getWe();

    // set header to never cache this response
    res.header('Cache-Control', 'private, no-cache, no-store, must-revalidate');
    res.header('Expires', '-1');
    res.header('Pragma', 'no-cache');

    var configs = we.getAppBootstrapConfig(we);

    return res.send(configs);
  },

  getTranslations: function (req, res) {
    var we = req.getWe();
    var localeParam = req.params.locale;
    var locale;

    if (localeParam) {
      // check if the locale are in avaible we.js locales
      // TODO add suport to search in subprojects
      for (var i = we.config.i18n.locales.length - 1; i >= 0; i--) {
        if ( we.config.i18n.locales[i] === localeParam ) {
          locale = localeParam;
        }
      }
    }

    // TODO change to use res.locals.locale
    if ( req.isAuthenticated()) {
      locale = req.user.language;
    }

    if (!locale) {
      locale = we.config.i18n.defaultLocale;
    }

    var translationResponse = '';

    translationResponse += 'if(typeof Ember === "undefined"){' +
      'Ember = {};' +
      'Ember.I18n = {};' +
    '}\n';

    translationResponse += 'if(!Ember.I18n.translations){' +
      'Ember.I18n.translations = {};' +
    '}\n';

    getTranslationFilePath(we, locale , function (path) {
      if (path) {
        fs.readFile(path, 'utf8', function (err, data) {
          if (err) {
            we.log.error('Error: ' + err);
            return res.serverError();
          }

          translationResponse += 'Ember.I18n.translations = ';
          translationResponse+= data;
          translationResponse += ';';

          res.contentType('application/javascript');
          res.ok( translationResponse );
        });
      } else {
        we.log.debug('getTranslations:Locale not found:', locale, localeParam);
        res.contentType('application/javascript');
        res.ok( translationResponse );
      }

    });

  },

  /**
   * Get all sails models converted to ember.js model
   *
   * TODO check if user is admin
   */
  getAllModelsAsEmberModel: function(req, res) {
    var we = req.getWe();

    // cache it in a static variable
    if ( !staticModels) {
      staticModels = converter.convertMultipleToEmberJSFile( we.db.modelsConfigs );
    }

    res.set('Content-Type', 'application/javascript');

    res.send(staticModels);
  },

  getRoutes: function(req, res) {
    var we = req.getWe();
    var getRoutes = {};
    var url;

    for(var routePath in we.routes) {
      if (routePath.substring(0, 4) === 'get ' && we.routes[routePath].responseType != 'json') {
        url = routePath.replace('get ', '');
        getRoutes[url] = we.routes[routePath];
      }
    }

    res.send(getRoutes);
  }
};

/**
 * Search for one locale file folder
 *
 * @param  {Object}   we       we.js
 * @param  {String}   locale   locale to search for
 * @param  {Function} callback
 */
function getTranslationFilePath (we, locale, callback) {
  var localePath = null;
  // check if exists in project
  localePath = we.projectPath + '/config/locales/' + locale + '.json';
  fs.exists (localePath, function (exists) {
    if (exists) {
      return callback(localePath);
    }

    we.log.info('Localization file not found in project', locale);

    callback();
  });
}

